@@ -1,6 +1,9 @@
 'use strict';
 
-var def = require('./def');
+var def = require('./def'),
+    estraverse = require('estraverse');
+
+var Syntax = estraverse.Syntax;
 
 function main (cxt) {
 
@@ -15,21 +18,21 @@ function main (cxt) {
             type: 'Program',
             body: [
                 {
-                    type: 'ExpressionStatement',
+                    type: Syntax.ExpressionStatement,
                     expression: {
-                        type: 'AssignmentExpression',
+                        type: Syntax.AssignmentExpression,
                         operator: '=',
                         left: {
-                            type: 'Identifier',
+                            type: Syntax.Identifier,
                             name: 'fn'
                         },
                         right: {
-                            type: 'FunctionExpression',
+                            type: Syntax.FunctionExpression,
                             id: null,
                             params: [],
                             defaults: [],
                             body: {
-                                type: 'BlockStatement',
+                                type: Syntax.BlockStatement,
                                 body: this.pos
                             },
                             generator: false,
@@ -62,7 +65,7 @@ Execution: ( -- ) ( R: nest-sys -- )
 
 def('exit', function () {
     this.pos.push({
-        type: 'ReturnStatement',
+        type: Syntax.ReturnStatement,
         argument: null
     });
 }, cxt, { immediate: true });
@@ -109,24 +112,24 @@ Run-time: ( x -- )
         this.cfs.push(oldpos);
         this.pos = [];
         oldpos.push({
-            type: 'IfStatement',
+            type: Syntax.IfStatement,
             test: {
-                type: 'CallExpression',
+                type: Syntax.CallExpression,
                 callee: {
-                    type: 'MemberExpression',
+                    type: Syntax.MemberExpression,
                     computed: false,
                     object: {
-                        type: 'ThisExpression'
+                        type: Syntax.ThisExpression
                     },
                     property: {
-                        type: 'Identifier',
+                        type: Syntax.Identifier,
                         name: 'dpop'
                     }
                 },
                 arguments: []
             },
             consequent: {
-                type: 'BlockStatement',
+                type: Syntax.BlockStatement,
                 body: this.pos
             },
             alternate: null
@@ -166,11 +169,18 @@ Compilation: ( C: orig1 -- orig2 )
 Run-time: ( -- )
     Continue execution at the location given by the resolution of orig2.
 */
+//  : ELSE ( orig1 -- orig2 / -- )
+//      \ resolve IF supplying alternate execution
+//      POSTPONE AHEAD  \ unconditional forward branch orig2
+//      1 CS-ROLL       \ put orig1 back on top
+//      POSTPONE THEN   \ resolve forward branch from orig1
+//  ; IMMEDIATE
+
     def('else', function () {
         var pos = [];
         var oldblock = this.cfs[this.cfs.length - 1];
         oldblock[oldblock.length - 1].alternate = {
-            type: 'BlockStatement',
+            type: Syntax.BlockStatement,
             body: pos
         };
         this.pos = pos;
@@ -219,13 +229,13 @@ Run-time: ( -- )
         this.cfs.push(oldpos);
         this.pos = [];
         oldpos.push({
-            type: 'DoWhileStatement',
+            type: Syntax.DoWhileStatement,
             body: {
-                type: 'BlockStatement',
+                type: Syntax.BlockStatement,
                 body: this.pos
             },
             test: {
-                type: 'Literal',
+                type: Syntax.Literal,
                 value: true,
                 raw: 'true'
             }
@@ -249,18 +259,18 @@ Run-time: ( x -- )
     def('until', function () {
         this.pos = this.cfs.pop();
         this.pos[this.pos.length - 1].test = {
-            type: 'UnaryExpression',
+            type: Syntax.UnaryExpression,
             operator: '!',
             argument: {
-                type: 'CallExpression',
+                type: Syntax.CallExpression,
                 callee: {
-                    type: 'MemberExpression',
+                    type: Syntax.MemberExpression,
                     computed: false,
                     object: {
-                        type: 'ThisExpression'
+                        type: Syntax.ThisExpression
                     },
                     property: {
-                        type: 'Identifier',
+                        type: Syntax.Identifier,
                         name: 'dpop'
                     }
                 },
@@ -286,22 +296,29 @@ Run-time: ( x -- )
     If all bits of x are zero, continue execution at the location specified by
     the resolution of orig.
 */
+
+//  : WHILE ( dest -- orig dest / flag -- )
+//      \ conditional exit from loops
+//      POSTPONE IF     \ conditional forward brach
+//      1 CS-ROLL       \ keep dest on top
+//  ; IMMEDIATE
+
     def('while', function () {
         this.pos.push({
-            type: 'IfStatement',
+            type: Syntax.IfStatement,
             test: {
-                type: 'UnaryExpression',
+                type: Syntax.UnaryExpression,
                 operator: '!',
                 argument: {
-                    type: 'CallExpression',
+                    type: Syntax.CallExpression,
                     callee: {
-                        type: 'MemberExpression',
+                        type: Syntax.MemberExpression,
                         computed: false,
                         object: {
-                            type: 'ThisExpression'
+                            type: Syntax.ThisExpression
                         },
                         property: {
-                            type: 'Identifier',
+                            type: Syntax.Identifier,
                             name: 'dpop'
                         }
                     },
@@ -310,10 +327,10 @@ Run-time: ( x -- )
                 prefix: true
             },
             consequent: {
-                type: 'BlockStatement',
+                type: Syntax.BlockStatement,
                 body: [
                     {
-                        type: 'BreakStatement',
+                        type: Syntax.BreakStatement,
                         label: null
                     }
                 ]
@@ -339,6 +356,12 @@ Run-time: ( -- )
         this.pos = this.cfs.pop();
     }, cxt, { immediate: true });
 
+//  : REPEAT ( orig dest -- / -- )
+//      \ resolve a single WHILE and return to BEGIN
+//      POSTPONE AGAIN  \ uncond. backward branch to dest
+//      POSTPONE THEN   \ resolve forward branch from orig
+//  ; IMMEDIATE
+
     def('repeat', function () {
         this.pos = this.cfs.pop();
     }, cxt, { immediate: true });
@@ -357,13 +380,50 @@ Run-time: ( -- )
     Continue execution at the location specified by the resolution of orig.
 */
 
-
+// : DO POSTPONE 2>R HERE ; IMMEDIATE
     def('do', function () {
     }, cxt, { immediate: true });
 
     def('loop', function () {
     }, cxt, { immediate: true });
 
+
+/*
+
+    6.1.2120 RECURSE (CORE)
+
+Interpretation: Interpretation semantics for this word are undefined.
+
+Compilation: ( -- )
+    Append the execution semantics of the current definition to the current
+    definition. An ambiguous condition exists if RECURSE appears in a definition
+    after DOES>.
+*/
+
+    def('recurse', function () {
+        var index;
+        index = this.L.length;
+        this.pos.push({
+            type: Syntax.ExpressionStatement,
+            expression: {
+                type: Syntax.CallExpression,
+                callee: {
+                    type: Syntax.MemberExpression,
+                    computed: true,
+                    object: {
+                        type: Syntax.ThisExpression
+                    },
+                    property: {
+                        type: Syntax.Literal,
+                        value: index,
+                        raw: index + ''
+                    }
+                },
+                arguments: []
+            }
+        });
+    }, cxt, { immediate: true });
+
 }
 
 module.exports = main;
