@@ -77,7 +77,7 @@ function interpret (str, done) {
                    this.pos.push(callword(wordLink.index));
                }
             } else {
-                num = Number.parseInt(w, this.base);
+                num = parseInt(w, this.base);
                 if (Number.isNaN(num)) {
                     console.log('word: ' + w + ' is not defined');
                     throw new Error;
@@ -89,7 +89,7 @@ function interpret (str, done) {
             if (wordLink) {
                 this[wordLink.index]();
             } else {
-                num = Number.parseInt(w, this.base);
+                num = parseInt(w, this.base);
                 if (Number.isNaN(num)) {
                     console.log('word: ' + w + ' is not defined');
                     throw new Error;
